/*
 *  Microfeed - Backend for accessing feed-based services (utils)
 *  Copyright (C) 2009 Henrik Hedberg <henrik.hedberg@innologies.fi>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "config.h"

#include <microfeed-common/microfeedconfiguration.h>
#include <microfeed-common/microfeedprotocol.h>
#include <microfeed-common/microfeedmisc.h>
#include <string.h>

#include "configuration.h"

typedef struct {
	GtkWidget* dialog;
	GtkWidget* subscribe_button;
	GtkWidget* edit_button;
	GtkWidget* delete_button;
} PublishersDialog;


static void publisher_destroyed(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const char* error_name, const char* error_message, void* user_data) {
	GtkWidget* dialog;

	if (error_name) {
		dialog = gtk_message_dialog_new(*((GtkWindow**)user_data), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Could not delete %s: %s", publisher, (error_message ? error_message : error_name));
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		if (*((GtkDialog**)user_data)) {
			gtk_dialog_response(*((GtkDialog**)user_data), GTK_RESPONSE_REJECT);
		}
	} else {
		if (*((GtkDialog**)user_data)) {
			gtk_dialog_response(*((GtkDialog**)user_data), GTK_RESPONSE_ACCEPT);
		}
	}
}

static void show_delete_dialog(MicrofeedSubscriber* subscriber, GtkWindow* parent, const char* publisher) {
	GtkWidget* dialog;
	GtkWidget* info_dialog;

	dialog = gtk_message_dialog_new(parent, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "Do you readlly want to delete the publisher called %s?", publisher);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES) {
		if (microfeed_subscriber_destroy_publisher(subscriber, publisher, publisher_destroyed, &info_dialog)) {
			info_dialog = gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CANCEL, "Deleting publisher... Wait!");
			gtk_dialog_run(GTK_DIALOG(info_dialog));
			gtk_widget_destroy(info_dialog);
		}
	}
	gtk_widget_destroy(dialog);
}

#ifdef MAEMO_CHANGES

static void on_changed(HildonTouchSelector* touch_selector, gint column, gpointer user_data) {
	PublishersDialog* publishers_dialog;
	gboolean selected;
	
	publishers_dialog = (PublishersDialog*)user_data;
	selected = (hildon_touch_selector_get_active(touch_selector, 0) == -1 ? FALSE : TRUE);
	if (publishers_dialog->subscribe_button) {
		gtk_widget_set_sensitive(publishers_dialog->subscribe_button, selected);
	}
	gtk_widget_set_sensitive(publishers_dialog->edit_button, selected);
	gtk_widget_set_sensitive(publishers_dialog->delete_button, selected);
}

#else

static void on_row_activated(GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gpointer user_data) {
	PublishersDialog* publishers_dialog;
	
	publishers_dialog = (PublishersDialog*)user_data;
	gtk_dialog_response(GTK_DIALOG(publishers_dialog->dialog), 2);
}

static void on_selection_changed(GtkTreeSelection* selection, gpointer user_data) {
	PublishersDialog* publishers_dialog;
	gboolean selected;

	publishers_dialog = (PublishersDialog*)user_data;
	selected = gtk_tree_selection_get_selected(selection, NULL, NULL);
	if (publishers_dialog->subscribe_button) {
		gtk_widget_set_sensitive(publishers_dialog->subscribe_button, selected);
	}
	gtk_widget_set_sensitive(publishers_dialog->edit_button, selected);
	gtk_widget_set_sensitive(publishers_dialog->delete_button, selected);
}

#endif

static void update_list(MicrofeedConfiguration* configuration, GtkListStore* list_store, const char* identifier) {
	const char** subscriptions;
	const char** publishers;
	GtkTreeIter iter;
	const char* separator;
	char* publisher;
	const char* name;
	char* label;
	const char** s;
	gboolean subscribed;

	gtk_list_store_clear(list_store);
	microfeed_configuration_invalidate(configuration);
	if (identifier) {
		subscriptions = microfeed_configuration_get_subscriptions(configuration, identifier);
	} else {
		subscriptions = NULL;
	}
	for (publishers = microfeed_configuration_get_publishers(configuration); publishers && *publishers; publishers++) {
		if ((separator = strchr(*publishers, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR))) {
			publisher = g_strndup(*publishers, separator - *publishers);
			name = microfeed_configuration_get_provider_name(configuration, separator + 1);
			label = microfeed_util_string_concatenate(publisher, " (", name, ")", NULL);
			g_free(publisher);
		} else {
			label = strdup(*publishers);
		}
		subscribed = FALSE;
		for (s = subscriptions; s && *s; s++) {
			if (!strcmp(*publishers, *s)) {
				subscribed = TRUE;
				break;
			}
		}
		gtk_list_store_append(list_store, &iter);
		gtk_list_store_set(list_store, &iter, 0, subscribed, 1, label, 2, *publishers, -1);
		free(label);
	}
}

void show_publishers_dialog(MicrofeedSubscriber* subscriber, GtkWindow* parent, const char* identifier) {
	MicrofeedConfiguration* configuration;
	PublishersDialog* publishers_dialog;
	GtkWidget* button;
	GtkListStore* list_store;
#ifdef MAEMO_CHANGES
	GtkWidget* touch_selector;
	HildonTouchSelectorColumn* column;
#else
	GtkWidget* tree_view;
	GtkWidget* scrolled_window;
	GtkTreeViewColumn* column;
	GtkTreeSelection* selection;
#endif
	GtkCellRenderer* renderer;
	GtkTreeIter iter;
	GValue value = { 0 };
	GValue value2 = { 0 };
	int response;
	char* publisher;

	configuration = microfeed_configuration_new();
	
	publishers_dialog = microfeed_memory_allocate(PublishersDialog);
	publishers_dialog->dialog = gtk_dialog_new_with_buttons((identifier ? "Subscribe Microfeed Publishers" : "Configure Microfeed Publishers"), parent, 0, NULL);
	button = gtk_button_new_from_stock(GTK_STOCK_NEW);
	gtk_dialog_add_action_widget(GTK_DIALOG(publishers_dialog->dialog), button, 1);
	if (identifier) {
		publishers_dialog->subscribe_button = gtk_button_new_with_label("(Un)Subscribe");
		gtk_widget_set_sensitive(publishers_dialog->subscribe_button, FALSE);
		gtk_dialog_add_action_widget(GTK_DIALOG(publishers_dialog->dialog), publishers_dialog->subscribe_button, 4);
	}
	publishers_dialog->edit_button = gtk_button_new_from_stock(GTK_STOCK_EDIT);
	gtk_widget_set_sensitive(publishers_dialog->edit_button, FALSE);
	gtk_dialog_add_action_widget(GTK_DIALOG(publishers_dialog->dialog), publishers_dialog->edit_button, 2);
#ifdef MAEMO_CHANGES
	publishers_dialog->delete_button = hildon_button_new_with_text(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL, "Delete", NULL);
#else
	publishers_dialog->delete_button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
#endif
	gtk_widget_set_sensitive(publishers_dialog->delete_button, FALSE);
	gtk_dialog_add_action_widget(GTK_DIALOG(publishers_dialog->dialog), publishers_dialog->delete_button, 3);
#ifndef MAEMO_CHANGES
	button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	gtk_dialog_add_action_widget(GTK_DIALOG(publishers_dialog->dialog), button, GTK_RESPONSE_CLOSE);
#endif

	list_store = gtk_list_store_new(3, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING);
	update_list(configuration, list_store, identifier);
	
#ifdef MAEMO_CHANGES
	touch_selector = hildon_touch_selector_new();
	g_signal_connect(touch_selector, "changed", G_CALLBACK(on_changed), publishers_dialog);
	column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(touch_selector), GTK_TREE_MODEL(list_store), NULL, NULL);

	if (identifier) {
		renderer = gtk_cell_renderer_toggle_new();
		g_object_set(renderer, "xpad", 8, NULL);
		gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, FALSE);
		gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(column), renderer, "active", 0);
	}
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(column), renderer, "text", 1);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(publishers_dialog->dialog)->vbox), touch_selector);
#else

	tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store));
	gtk_widget_set_size_request(tree_view, 400, 150);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), FALSE);
	g_signal_connect(tree_view, "row-activated", G_CALLBACK(on_row_activated), publishers_dialog);
	g_object_unref(G_OBJECT(list_store));

	if (identifier) {
		renderer = gtk_cell_renderer_toggle_new();
		column = gtk_tree_view_column_new_with_attributes("Subscribe", renderer, "active", 0, NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
	}
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Publisher", renderer, "text", 1, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), GTK_SHADOW_IN);
 	gtk_container_add(GTK_CONTAINER(scrolled_window), tree_view);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(publishers_dialog->dialog)->vbox), scrolled_window);
			
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	g_signal_connect(selection, "changed", G_CALLBACK(on_selection_changed), publishers_dialog);
#endif
			
	gtk_widget_show_all(publishers_dialog->dialog);
	
	while ((response = gtk_dialog_run(GTK_DIALOG(publishers_dialog->dialog))) != GTK_RESPONSE_DELETE_EVENT && response != GTK_RESPONSE_CLOSE) {
		if (response == 1) {
			if ((publisher = show_new_dialog(subscriber, GTK_WINDOW(publishers_dialog->dialog)))) {
				if (identifier) {
					microfeed_configuration_add_subscription(configuration, identifier, publisher);
				}
				update_list(configuration, list_store, identifier);
				show_edit_dialog(subscriber, GTK_WINDOW(publishers_dialog->dialog), publisher, 1);	
				free(publisher);
			}
		} else if (response == 2) {
#ifdef MAEMO_CHANGES
			if (hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(touch_selector), 0, &iter)) {
#else
			if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
#endif
				gtk_tree_model_get_value(GTK_TREE_MODEL(list_store), &iter, 2, &value);
				show_edit_dialog(subscriber, GTK_WINDOW(publishers_dialog->dialog), g_value_get_string(&value), 0);
				g_value_unset(&value);
			}
		} else if (response == 3) {
#ifdef MAEMO_CHANGES
			if (hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(touch_selector), 0, &iter)) {
#else
			if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
#endif
				gtk_tree_model_get_value(GTK_TREE_MODEL(list_store), &iter, 2, &value);
				show_delete_dialog(subscriber, GTK_WINDOW(publishers_dialog->dialog), g_value_get_string(&value));
				g_value_unset(&value);
				update_list(configuration, list_store, identifier);
				
			}
		} else if (response == 4) {
#ifdef MAEMO_CHANGES
			if (hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(touch_selector), 0, &iter)) {
#else
			if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
#endif
				gtk_tree_model_get_value(GTK_TREE_MODEL(list_store), &iter, 0, &value);
				gtk_tree_model_get_value(GTK_TREE_MODEL(list_store), &iter, 2, &value2);
				if (g_value_get_boolean(&value)) {
					microfeed_configuration_remove_subscription(configuration, identifier, g_value_get_string(&value2));
				} else {
					microfeed_configuration_add_subscription(configuration, identifier, g_value_get_string(&value2));
				}
				g_value_set_boolean(&value, !g_value_get_boolean(&value));
				gtk_list_store_set_value(list_store, &iter, 0, &value);
				g_value_unset(&value);
			}
		}
	}
	
	gtk_widget_destroy(publishers_dialog->dialog);
	microfeed_configuration_free(configuration);
}
